import pytest

pytestmark = pytest.mark.sphinx('html', testroot='genericpage3')


@pytest.mark.parametrize('page', ['about.html', ], indirect=True)
class TestGenericpage3:

    def test_about(self, page):
        # The root's acquireds has a all: template pointing to
        # acquired_all. The _templates dir has a
        # acquired_all.html template. We should match in that.
        content = page.find('h1').contents[0].strip()
        assert 'Using acquired_all' == content


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestGenericpage3Debug:
    # Get template from the acquireds -> genericpage

    def test_settings(self, json_page):
        assert 'genericpages' in json_page
        genericpages = json_page['genericpages']
        assert 'config' in genericpages

        # genericpage3 contains no registered handlers
        config = genericpages['config']
        assert {} == config

        # one value in genericpage, the 'about' document
        values = genericpages['values']
        assert 1 == len(values)
        assert 'about' in values
        about = values['about']
        assert 'about' == about['docname']
        assert 'acquired_all' == about['template']



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='acquire')


@pytest.mark.parametrize('page', ['folder1/about.html', ], indirect=True)
class TestAcquire:

    def test_about(self, page):
        # First, make sure we are in the right place
        h1 = page.find('h1').contents[0].strip()
        assert 'Root Article' == h1

        # Test that resource.acquire worked
        template = page.find(id='template').contents[0].strip()
        assert 'Acquire: root_article' == template
        flag = page.find(id='flag').contents[0].strip()
        assert 'Acquire: 99' == flag


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestAcquireDebug:

    @pytest.mark.parametrize('docname, template', [
        ('index', 'aqhomepage'),
        ('about', 'aqarticle'),
        ('folder1/index', 'folder1_section'),
        ('folder1/about', 'root_article'),
        ('folder1/subfolder2/index', 'subfolder2_section'),
        ('folder1/subfolder2/about', 'root_article'),
    ])
    def test_page(self, json_page, docname, template):
        resources = json_page['resources']['values']
        resource = resources[docname]
        assert template == resource['template']



from pathlib import Path

import pytest

pytestmark = pytest.mark.sphinx('html', testroot='imagemodel1')


@pytest.mark.parametrize('page', ['article1.html', ], indirect=True)
class TestImageModels1:

    def test_article1(self, page, content):
        p = Path(content.outdir, 'unittest.png')
        assert p.exists()
        h1 = page.find('h1').contents[0].strip()
        assert 'Article 1' == h1

        usage = page.find(class_='kb-images-usage').contents[0].strip()
        assert 'teaser' == usage



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='resources')


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestResourceDebug:

    def test_settings(self, json_page):
        assert 'resources' in json_page
        settings = json_page['resources']
        assert 'config' in settings
        assert 'values' in settings

        # Inspect some details from a value
        index = settings['values']['index']
        assert 'index' == index['docname']
        assert 'Hello World' == index['title']
        assert 'page' == index['template']
        assert 'resource' == index['rtype']
        assert None is index['parent']
        assert 'index' == index['repr']



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='references1')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestReferences1:

    def test_index(self, page):
        h1 = page.find('h1').contents[0].strip()
        assert 'Indexpage' == h1
        li = page.find('li').contents[0].strip()
        assert 'reference1' == li

        # Let's look at the two types of generated reference links
        refs = page.find_all("a", class_="internal")
        assert 3 == len(refs)  # Includes toctree on
        derived_title = refs[0].find('em').contents[0].strip()
        assert 'Reference 1' == derived_title
        explicit_title = refs[1].find('em').contents[0].strip()
        assert 'Reference 1-ish' == explicit_title


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestReferences1Debug:

    def test_settings(self, json_page):
        assert 'references' in json_page
        references = json_page['references']
        assert 'config' in references

        # references1 contains one registered handler
        config = references['config']
        v = config['reference']
        assert 'kaybee.plugins.references.reference.Reference' == v

        # one value in references
        values = references['values']
        assert 3 == len(values)
        assert 'reference' in values
        reference = values['reference']
        assert 'reference1' in reference
        assert 'reference1' == reference['reference1']['docname']

        # Let's see on the other side with the source of the reference
        resource = json_page['resources']['values']['index']
        assert 'reference1' == resource['props']['reference'][0]



"""

Dectate action to manage event callbacks in the configuration.

"""

import importlib
import os
import sys
from enum import Enum
from typing import List

import dectate
import importscan
from docutils.readers import doctree
from sphinx.application import Sphinx
from sphinx.builders.html import StandaloneHTMLBuilder
from sphinx.environment import BuildEnvironment
from sphinx.util import logging

logger = logging.getLogger(__name__)


class SphinxEvent(Enum):
    BI = 'builder-inited'
    EPD = 'env-purge-doc'
    EBRD = 'env-before-read-docs'
    DREAD = 'doctree-read'
    DRES = 'doctree-resolved'
    MR = 'missing-reference'
    HCP = 'html-collect-pages'
    EU = 'env-updated'
    ECC = 'env-check-consistency'
    HPC = 'html-page-context'


class EventAction(dectate.Action):
    config = {
        'events': dict
    }

    def __init__(self,
                 name: str,
                 order: int = 20,
                 scope=None,
                 system_order=None):
        assert name in SphinxEvent
        super().__init__()
        self.name = name
        self.scope = scope

        if system_order is None:
            # This is a user handler
            # 40 to 80 is reserved for system handlers.
            assert order < 40 or order > 80
            self.order = order
        else:
            assert 40 <= system_order <= 80
            self.order = system_order

    def identifier(self, events):
        if self.scope:
            return f'{self.name.value}-{self.scope}-{self.order}'
        else:
            return f'{self.name.value}-{self.order}'

    # noinspection PyMethodOverriding
    def perform(self, obj, events):
        events[self.name] = obj

    @classmethod
    def get_callbacks(cls, registry, event_name: SphinxEvent):
        # Presumes the registry has been committed

        # First ensure that event_name is valid
        assert event_name in SphinxEvent

        q = dectate.Query('event')
        qr = sorted(q(registry), key=lambda args: args[0].order)
        return [args[1] for args in qr if args[0].name == event_name]

    #
    # Dispatchers for each of the Sphinx methods
    #
    @classmethod
    def call_builder_init(cls, kb_app, sphinx_app: Sphinx):
        """ On builder init event, commit registry and do callbacks """

        # Find and commit docs project plugins
        conf_dir = sphinx_app.confdir
        plugins_dir = sphinx_app.config.kaybee_settings.plugins_dir
        full_plugins_dir = os.path.join(conf_dir, plugins_dir)

        if os.path.exists(full_plugins_dir):
            sys.path.insert(0, conf_dir)
            plugin_package = importlib.import_module(plugins_dir)
            importscan.scan(plugin_package)
        else:
            logger.info(f'## Kaybee: No plugin dir at {plugins_dir}')

        dectate.commit(kb_app)
        for callback in cls.get_callbacks(kb_app, SphinxEvent.BI):
            callback(kb_app, sphinx_app)

    @classmethod
    def call_purge_doc(cls, kb_app, sphinx_app: Sphinx,
                       sphinx_env: BuildEnvironment,
                       docname: str):
        """ On env-purge-doc, do callbacks """

        for callback in EventAction.get_callbacks(kb_app, SphinxEvent.EPD):
            callback(kb_app, sphinx_app, sphinx_env, docname)

    @classmethod
    def call_env_before_read_docs(cls, kb_app, sphinx_app: Sphinx,
                                  sphinx_env: BuildEnvironment,
                                  docnames: List[str]):
        """ On env-read-docs, do callbacks"""

        for callback in EventAction.get_callbacks(kb_app,
                                                  SphinxEvent.EBRD):
            callback(kb_app, sphinx_app, sphinx_env, docnames)

    @classmethod
    def call_env_doctree_read(cls, kb_app, sphinx_app: Sphinx,
                              doctree: doctree):
        """ On doctree-read, do callbacks"""

        for callback in EventAction.get_callbacks(kb_app,
                                                  SphinxEvent.DREAD):
            callback(kb_app, sphinx_app, doctree)

    @classmethod
    def call_doctree_resolved(cls, kb_app, sphinx_app: Sphinx,
                              doctree: doctree,
                              fromdocname: str):
        """ On doctree-resolved, do callbacks"""

        for callback in EventAction.get_callbacks(kb_app,
                                                  SphinxEvent.DRES):
            callback(kb_app, sphinx_app, doctree, fromdocname)

    @classmethod
    def call_env_updated(cls, kb_app,
                         sphinx_app: Sphinx,
                         sphinx_env: BuildEnvironment):

        """ On the env-updated event, do callbacks """
        for callback in EventAction.get_callbacks(kb_app, SphinxEvent.EU):
            callback(kb_app, sphinx_app, sphinx_env)

    @classmethod
    def call_html_collect_pages(cls, kb_app, sphinx_app: Sphinx):
        """ On html-collect-pages, do callbacks"""

        EventAction.get_callbacks(kb_app,
                                  SphinxEvent.HCP)
        for callback in EventAction.get_callbacks(kb_app,
                                                  SphinxEvent.HCP):
            yield callback(kb_app, sphinx_app)

    @classmethod
    def call_env_check_consistency(cls, kb_app, builder: StandaloneHTMLBuilder,
                                   sphinx_env: BuildEnvironment):
        """ On env-check-consistency, do callbacks"""

        for callback in EventAction.get_callbacks(kb_app,
                                                  SphinxEvent.ECC):
            callback(kb_app, builder, sphinx_env)

    @classmethod
    def call_missing_reference(cls, kb_app, sphinx_app: Sphinx,
                               sphinx_env: BuildEnvironment,
                               node,
                               contnode,
                               ):
        """ On doctree-resolved, do callbacks"""

        for callback in EventAction.get_callbacks(kb_app,
                                                  SphinxEvent.MR):
            return callback(kb_app, sphinx_app, sphinx_env, node, contnode)

    @classmethod
    def call_html_page_context(cls, kb_app, sphinx_app: Sphinx,
                               pagename: str,
                               templatename: str,
                               context,
                               doctree: doctree
                               ):
        """ On doctree-resolved, do callbacks"""

        # We need to let one, and only one, callback return the name of
        # the template. Detect multiple and raise an exception.
        new_templatename = None

        for callback in EventAction.get_callbacks(kb_app,
                                                  SphinxEvent.HPC):
            # The protocol: the one controlling callback will return a value
            # with a dictionary of {'templatename': 'sometemplate'}
            result = callback(kb_app, sphinx_app, pagename, templatename,
                              context,
                              doctree)
            if result and isinstance(result,
                                     dict) and 'templatename' in result:
                if new_templatename is not None:
                    raise AssertionError('Multiple handlers returning')
                new_templatename = result['templatename']

        return new_templatename



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='settings')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestSettingsIndex:

    def test_index(self, page):
        h1 = page.find('h1').contents[0].strip()
        assert 'Settings Template' == h1
        div = page.find('div').contents[0].strip()
        assert 'True' == div


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestSettingsDebug:

    def test_settings(self, json_page):
        assert 'use_debug' in json_page['settings']['debugdumper']



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='articles1')


@pytest.mark.parametrize('page', ['articles/article1.html', ], indirect=True)
class TestArticles1:

    def test_article1(self, page):
        h1 = page.find('h1').contents[0].strip()
        assert 'Article 1' == h1


@pytest.mark.parametrize('page', ['categories/category2.html', ],
                         indirect=True)
class TestCategories2:

    def test_article2(self, page):
        h1 = page.find('h1').contents[0].strip()
        assert 'Category 2' == h1

        ul = page.find(id='kb-reference-listing')
        li = ul.find_all('li')
        items = [i.contents[0].strip()
                 for i in page.find_all(class_='kb-reference-item')]
        assert 2 == len(items)
        assert 'Article 1' in items


@pytest.mark.parametrize('page', ['articles/article2.html', ],
                         indirect=True)
class TestQuerylist:

    def test_querylist(self, page):
        h1 = page.find('h1').contents[0].strip()
        assert 'Article 2' == h1

        labels = [
            result.contents[0].strip()
            for result in page.find_all(class_='kb-querylist-label')]
        items = [
            result.contents[0].strip()
            for result in page.find_all(class_='kb-querylist-item')]
        assert 2 == len(labels)
        assert 'Recent Blog Posts' in labels
        assert 6 == len(items)
        assert 'Section 1' in items


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestArticles1Debug:

    def test_settings(self, json_page):
        resources = json_page['resources']
        assert 'config' in resources

        # article/section/homepage is in the registered handlers
        resources_config = resources['config']
        assert 'article' in resources_config
        assert 'homepage' in resources_config
        assert 'section' in resources_config

        # Test some values
        resource_values = resources['values']

        # homepage
        homepage = resource_values['index']
        assert 'index' == homepage['docname']
        assert 'Hello World' == homepage['title']
        assert '' == homepage['section']
        assert 'articles/index' in homepage['toctree']
        assert None is homepage['series']

        # articles
        articles = resource_values['articles/index']
        assert 'articles/index' == articles['docname']
        assert 'Articles' == articles['title']
        assert '' == articles['section']
        assert None is articles['series']

        # article1
        article1 = resource_values['articles/article1']
        assert 'articles/article1' == article1['docname']
        assert 'Article 1' == article1['title']
        assert 'articles/index' == article1['section']
        assert [] == article1['toctree']
        assert 3 == len(article1['series'])

        # section1
        section1 = resource_values['section1/index']
        assert 'section1/index' == section1['docname']
        assert 'Section 1' == section1['title']
        assert '' == section1['section']
        assert ['section1/article2'] == section1['toctree']
        assert 'section1/article2' == section1['get_featured_resource']



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='toctrees1')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestToctrees1:

    def test_index(self, page):
        h1 = page.find('h1').contents[0].strip()
        assert 'Custom Homepage' == h1

        entries = page.find_all("a", class_="reference")
        assert 2 == len(entries)
        assert 'article1.html' == entries[0].attrs['href'].strip()
        assert 'section1/index.html' == entries[1].attrs['href'].strip()


@pytest.mark.parametrize('page', ['section1/index.html', ], indirect=True)
class TestToctrees1:

    def test_section1(self, page):
        div = page.find(id='first_toctree_title').contents[0].strip()
        # The resource only knows about the docname in the toctree. The
        # article.series is what converts to resources.
        assert 'section1/article2' == div


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestToctrees1Debug:

    def test_settings(self, json_page):
        resources = json_page['resources']
        assert 'config' in resources

        # article/section/homepage is in the registered handlers
        resources_config = resources['config']
        assert 'article' in resources_config
        assert 'homepage' in resources_config
        assert 'section' in resources_config

        # toctrees are registered (built-in and one custom)
        toctrees = json_page['toctrees']
        toctrees_config = toctrees['config']
        assert 'None-80' in toctrees_config
        assert 'None-70' in toctrees_config
        assert 'Toctree' in toctrees_config['None-70']



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='postrenderer')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestPostrenderer:

    def test_prindex(self, page):
        h1 = page.find('h1').contents[0].strip()
        assert 'HELLO WORLD' == h1



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='sphinxapp')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestSphinxApp:

    def test_index(self, page):
        content = page.find('p').contents[0].strip()
        assert 'html' == content



"""

Built-in registry app that can be used directly in docs projects.

The app defines a set of configured actions. These come mostly from
plugins. Kaybee ships with some built-in plugins, and this app bundles
them up.

Other sites might have other plugins and thus need a custom subclass for
the app.

"""
import dectate

from kaybee.plugins.articles.actions import ToctreeAction
from kaybee.plugins.debugdumper.action import DumperAction
from kaybee.plugins.events import EventAction
from kaybee.plugins.genericpage.action import GenericpageAction
from kaybee.plugins.postrenderer.action import PostrendererAction
from kaybee.plugins.resources.action import ResourceAction
from kaybee.plugins.widgets.action import WidgetAction


class kb(dectate.App):
    event = dectate.directive(EventAction)
    dumper = dectate.directive(DumperAction)
    resource = dectate.directive(ResourceAction)
    genericpage = dectate.directive(GenericpageAction)
    widget = dectate.directive(WidgetAction)
    toctree = dectate.directive(ToctreeAction)
    postrenderer = dectate.directive(PostrendererAction)




import dectate
import importscan
from sphinx.application import Sphinx

from kaybee import plugins
from kaybee.app import kb
from kaybee.plugins.events import EventAction, SphinxEvent
from kaybee.plugins.settings.model import KaybeeSettings

__version__ = '0.1.3'
__title__ = "kaybee"


def flush_everything(sphinx_app, sphinx_env):
    e = sphinx_app.env
    values = list(e.resources.values()) + list(e.genericpages.values())
    return [value.docname for value in values]


def setup(app: Sphinx):
    """ Initialize Kaybee as a Sphinx extension """

    # Scan for directives, first in the system, second in the docs project
    importscan.scan(plugins)
    dectate.commit(kb)

    app.add_config_value('kaybee_settings', KaybeeSettings(), 'html')
    bridge = 'kaybee.plugins.postrenderer.config.KaybeeBridge'
    app.config.template_bridge = bridge

    app.connect('env-updated', flush_everything)
    app.connect(SphinxEvent.BI.value,
                # pragma nocover
                lambda sphinx_app: EventAction.call_builder_init(
                    kb, sphinx_app)
                )
    app.connect(SphinxEvent.EPD.value,
                # pragma nocover
                lambda sphinx_app, sphinx_env,
                       docname: EventAction.call_purge_doc(
                    kb, sphinx_app, sphinx_env, docname)
                )

    app.connect(SphinxEvent.EBRD.value,
                # pragma nocover
                lambda sphinx_app, sphinx_env,
                       docnames: EventAction.call_env_before_read_docs(
                    kb, sphinx_app, sphinx_env, docnames)
                )

    app.connect(SphinxEvent.DREAD.value,
                # pragma nocover
                lambda sphinx_app,
                       doctree: EventAction.call_env_doctree_read(
                    kb, sphinx_app, doctree)
                )

    app.connect(SphinxEvent.DRES.value,
                # pragma nocover
                lambda sphinx_app, doctree,
                       fromdocname: EventAction.call_doctree_resolved(
                    kb, sphinx_app, doctree, fromdocname)
                )

    app.connect(SphinxEvent.EU.value,
                # pragma nocover
                lambda sphinx_app, sphinx_env: EventAction.call_env_updated(
                    kb, sphinx_app, sphinx_env)
                )

    app.connect(SphinxEvent.HCP.value,
                # pragma nocover
                lambda sphinx_app: EventAction.call_html_collect_pages(
                    kb, sphinx_app)
                )

    app.connect(SphinxEvent.ECC.value,
                # pragma nocover
                lambda sphinx_builder,
                       sphinx_env: EventAction.call_env_check_consistency(
                    kb, sphinx_builder, sphinx_env)
                )

    app.connect(SphinxEvent.MR.value,
                # pragma nocover
                lambda sphinx_app, sphinx_env, node,
                       contnode: EventAction.call_missing_reference(
                    kb, sphinx_app, sphinx_env, node, contnode)
                )

    app.connect(SphinxEvent.HPC.value,
                # pragma nocover
                lambda sphinx_app, pagename, templatename, context,
                       doctree: EventAction.call_html_page_context(
                    kb, sphinx_app, pagename, templatename, context, doctree)
                )

    return dict(
        version=__version__,
        parallel_read_safe=False
    )



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='genericpage1')


@pytest.mark.parametrize('page', ['about.html', ], indirect=True)
class TestGenericpage1:

    def test_about(self, page):
        # The genericpage doesn't have an acquire for template, so just use
        # the normal page.html, which puts the doc title in the <h1>
        content = page.find('h1').contents[0].strip()
        assert 'About' == content


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestGenericpage1Debug:

    def test_settings(self, json_page):
        assert 'genericpages' in json_page
        genericpages = json_page['genericpages']
        assert 'config' in genericpages

        # genericpage1 contains no registered handlers
        config = genericpages['config']
        assert {} == config

        # one value in genericpage
        values = genericpages['values']
        assert 1 == len(values)
        assert 'about' in values
        about = values['about']
        assert 'about' == about['docname']
        assert 'page' == about['template']



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='events')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestEvents:

    def test_events(self, page):
        title = page.find('title').contents[0].strip()
        assert 'index' == title

        bi = page.find(id='bi').contents[0].strip()
        assert 'BI' == bi

        hpc = page.find(id='hpc').contents[0].strip()
        assert 'HPC' == hpc



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='kitchensink')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
def test_homepage(page):
    h1 = page.find('h1').contents[0].strip()
    assert 'Kitchen Sink' == h1

    kb_body = page.find(id='kb-body')
    nav = kb_body.find_all(class_='kb-navmenu-item')

    # kb-body stuff
    assert 'homepage' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert 'index' == kb_body.find(id='kb-docname').contents[0].strip()
    assert 'Kitchen Sink' == kb_body.find(id='kb-title').contents[0].strip()

    # Nav menu
    navitems = [navitem.contents[0].strip() for navitem in nav]
    assert 'Contact' in navitems
    assert 'About Us' in navitems
    assert 'Intro to Django' in navitems
    assert 'Hidden' not in navitems

    # Toctree
    toctree = page.find_all(class_='kb-toctree')
    assert 1 == len(toctree)
    anchors = [
        (a.attrs['href'], a.contents[0].strip())
        for a in toctree[0].find_all(class_='reference internal')
    ]
    assert 4 == len(anchors)
    assert 'hidden.html' in anchors[0][0]
    assert 'Hidden' == anchors[0][1]
    assert 'categories/index.html' in anchors[1][0]
    assert 'Category Listing' == anchors[1][1]

    # Featuretiles
    featuretiles = page.find_all(class_='kbb-featuretiles-tile')
    assert 6 == len(featuretiles)
    assert 'First Feature' == featuretiles[0].contents[0].strip()


@pytest.mark.parametrize('page', ['2018/intro_django.html', ], indirect=True)
def test_article(page):
    h1 = page.find('h1').contents[0].strip()
    assert 'Intro to Django' == h1

    # videoplayer
    vp = page.find(class_='kb-videoplayer')
    width = vp.attrs['width']
    assert '640' == width

    # Category listing
    categories = [i.contents[0].strip()
                  for i in page.find_all(class_='kb-category-item')]
    assert 'Django' in categories
    assert 'PostgreSQL' in categories

    # Inline references
    refs = page.find_all("a", class_="internal")
    derived_title = refs[0].find('em').contents[0].strip()
    assert 'Django' == derived_title
    explicit_title = refs[1].find('em').contents[0].strip()
    assert 'Django 2.0' == explicit_title

    # kb-body stuff
    kb_body = page.find(id='kb-body')
    assert 'article' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert '2018/intro_django' == kb_body.find(id='kb-docname').contents[
        0].strip()
    assert 'Intro to Django' == kb_body.find(id='kb-title').contents[0].strip()


@pytest.mark.parametrize('page', ['builtin_references/builtinref1.html', ],
                         indirect=True)
def test_builtinreference1(page):
    # Using the built-in reference type
    h1 = page.find('h1').contents[0].strip()
    assert 'Builtin Ref 1' == h1

    # kb-body stuff...doesn't exist, this is page.html
    kb_body = page.find(id='kb-body')
    assert None is kb_body


@pytest.mark.parametrize('page', ['builtin_references/builtinref2.html', ],
                         indirect=True)
def test_builtinreference2(page):
    # Using the built-in reference type
    h1 = page.find('h1').contents[0].strip()
    assert 'Builtin Ref 2' == h1

    label = page.find(id='kb-reference-label').contents[0].strip()
    assert 'builtinref2' == label

    # kb-body stuff
    kb_body = page.find(id='kb-body')
    assert 'reference' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert 'builtin_references/builtinref2' == \
           kb_body.find(id='kb-docname').contents[
               0].strip()
    assert 'Builtin Ref 2' == kb_body.find(id='kb-title').contents[0].strip()


@pytest.mark.parametrize('page', ['categories/django.html', ], indirect=True)
def test_category(page):
    h1 = page.find('h1').contents[0].strip()
    assert 'Django' == h1

    # Target references listing
    references = [i.contents[0].strip()
                  for i in page.find_all(class_='kb-reference-item')]
    assert 'Intro to Django' in references

    # kb-body stuff
    kb_body = page.find(id='kb-body')
    assert 'category' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert 'categories/django' == kb_body.find(id='kb-docname').contents[
        0].strip()
    assert 'Django' == kb_body.find(id='kb-title').contents[0].strip()


@pytest.mark.parametrize('page', ['customcategories/databases.html', ],
                         indirect=True)
def test_custom_category(page):
    h1 = page.find('h1').contents[0].strip()
    assert 'Databases' == h1

    # Target references listing
    references = [i.contents[0].strip()
                  for i in page.find_all(class_='kb-reference-item')]
    assert 'KSArticle 1' in references

    # kb-body stuff
    kb_body = page.find(id='kb-body')
    assert 'ksfeature' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert 'customcategories/databases' == \
           kb_body.find(id='kb-docname').contents[
               0].strip()
    assert 'Databases' == kb_body.find(id='kb-title').contents[0].strip()

    # Test the Jinja2 filter for date handling
    dtf = page.find(id='kb-datetime_filter').contents[0].strip()
    assert 'Oct 01' == dtf


@pytest.mark.parametrize('page', ['2018/index.html', ], indirect=True)
def test_section(page):
    h1 = page.find('h1').contents[0].strip()
    assert '2018 Articles' == h1

    # Querylist widget
    labels = [i.contents[0].strip()
              for i in page.find_all(class_='kb-querylist-label')]
    assert 'Recent Sections' in labels
    assert 'Recent Articles' in labels

    results = [i.contents[0].strip()
               for i in page.find_all(class_='kb-querylist-item')]
    assert '2018 Articles' in results
    assert 'About Us' in results

    # kb-body stuff
    kb_body = page.find(id='kb-body')
    assert 'section' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert '2018/index' == kb_body.find(id='kb-docname').contents[
        0].strip()
    assert '2018 Articles' == kb_body.find(id='kb-title').contents[0].strip()


@pytest.mark.parametrize('page', ['2017/index.html', ], indirect=True)
def test_sectionquery(page):
    h1 = page.find('h1').contents[0].strip()
    assert '2017 Articles' == h1

    # Sectionquery widget
    results = [i.contents[0].strip()
               for i in page.find_all(class_='kb-sectionquery-item')]
    assert ['KSArticle 1', 'KSArticle 2'] == results

    # kb-body stuff
    kb_body = page.find(id='kb-body')
    assert 'section' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert '2017/index' == kb_body.find(id='kb-docname').contents[
        0].strip()
    assert '2017 Articles' == kb_body.find(id='kb-title').contents[0].strip()


@pytest.mark.parametrize('page', ['builtin_references/index.html', ],
                         indirect=True)
def test_genericpage(page):
    # These are controlled from an acquireds setting on
    # index.rst
    h1 = page.find('h1').contents[0].strip()
    assert 'Builtin References' == h1

    assert 'builtin_references/index' == \
           page.find(id='kb-genericpage-docname').contents[0].strip()
    assert 'kitchensink_genericpage' == \
           page.find(id='kb-genericpage-template').contents[0].strip()


@pytest.mark.parametrize('page', ['2018/ksresource1.html', ], indirect=True)
def test_ksresource(page):
    h1 = page.find('h1').contents[0].strip()
    assert 'KSResource 1' == h1

    flag = page.find(id='kb-ksresource-flag').contents[0].strip()
    assert '9' == flag
    increment = page.find(id='kb-ksresource-increment').contents[0].strip()
    assert '10' == increment

    # kb-body stuff
    kb_body = page.find(id='kb-body')
    assert 'ksresource' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert '2018/ksresource1' == kb_body.find(id='kb-docname').contents[
        0].strip()
    assert 'KSResource 1' == kb_body.find(id='kb-title').contents[0].strip()


@pytest.mark.parametrize('page', ['2018/ksresource1.html', ], indirect=True)
def test_kswidget(page):
    h1 = page.find('h1').contents[0].strip()
    assert 'KSResource 1' == h1

    flag = page.find(id='kb-kswidget-flag').contents[0].strip()
    assert '9' == flag
    increment = page.find(id='kb-kswidget-increment').contents[0].strip()
    assert '10' == increment
    another_flag = page.find(id='kb-kswidget-anotherflag').contents[0].strip()
    assert '835' == another_flag


@pytest.mark.parametrize('page', ['2018/ksarticle1.html', ], indirect=True)
def test_ksarticle(page):
    h1 = page.find('h1').contents[0].strip()
    assert 'KSArticle 1' == h1

    flag = page.find(id='kb-ksarticle-flag').contents[0].strip()
    assert '9' == flag
    increment = page.find(id='kb-ksarticle-increment').contents[0].strip()
    assert '10' == increment

    # kb-body stuff
    kb_body = page.find(id='kb-body')
    assert 'ksarticle' == kb_body.find(id='kb-rtype').contents[0].strip()
    assert '2018/ksarticle1' == kb_body.find(id='kb-docname').contents[
        0].strip()
    assert 'KSArticle 1' == kb_body.find(id='kb-title').contents[0].strip()


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestResourceDebug:

    def test_settings(self, json_page):
        assert 'use_debug' in json_page['settings']['debugdumper']



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='setup')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestSetupFunction:

    def test_index(self, page):
        content = page.find('h1').contents[0].strip()
        assert 'Hello World' == content



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='debugdumper')


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestDebugDumper:

    def test_testdumper(self, json_page):
        assert '2017' in json_page['then']



import kaybee.plugins.articles.article
import kaybee.plugins.articles.featuretiles
import kaybee.plugins.articles.handlers
import kaybee.plugins.articles.homepage
import kaybee.plugins.articles.querylist
import kaybee.plugins.articles.category
import kaybee.plugins.articles.section
import kaybee.plugins.articles.sectionquery
import kaybee.plugins.articles.sectionbox
import kaybee.plugins.articles.toctree
import kaybee.plugins.articles.videoplayer
import kaybee.plugins.debugdumper.handlers
import kaybee.plugins.genericpage.handlers
import kaybee.plugins.localtemplates.handlers
import kaybee.plugins.queries.handlers
import kaybee.plugins.references.handlers
import kaybee.plugins.references.reference
import kaybee.plugins.resources.handlers
import kaybee.plugins.resources.resource
import kaybee.plugins.settings.handlers
import kaybee.plugins.sphinx_app.handlers
import kaybee.plugins.widgets.handlers
import kaybee.plugins.widgets.widget



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='widgets2')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestWidgets2:

    def test_index(self, page):
        # The genericpage doesn't have an acquire for template, so just use
        h1 = page.find('h1').contents[0].strip()
        assert 'Hello World' == h1

        hello = page.find(id='hello').contents[0].strip()
        assert 'Hello Custom Widget Template' == hello
        docname = page.find(id='resource_docname').contents[0].strip()
        assert 'index' == docname

        listing_flag = page.find(id='listing_flag').contents[0].strip()
        assert '98' == listing_flag
        another_flag = page.find(id='another_flag').contents[0].strip()
        assert '835' == another_flag
        widget_content = str(page.find(id='widget_content'))
        assert '<p>Here is some <em>more</em> text.</p>' in widget_content
        widget_greeting = page.find(id='widget_greeting').contents[0].strip()
        assert 'widget greeting' == widget_greeting
        resources = page.find(id='resources').contents[0].strip()
        assert 'resource' == resources
        resource = page.find(id='resource').contents[0].strip()
        assert 'index' == resource


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestWidgets2Debug:

    def test_settings(self, json_page):
        assert 'widgets' in json_page
        widgets = json_page['widgets']
        assert 'config' in widgets

        # widgets2 contains one registered handler
        config = widgets['config']
        assert 'listing' in config

        # one value in widgets
        values = widgets['values']
        assert 1 == len(values)
        assert 'index-widgets1hello' in values
        index = values['index-widgets1hello']
        assert 'index' == index['docname']
        assert 'props' in index
        props = index['props']
        assert None is props['template']



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='kaybee',
    version='0.1.3',
    description='Knowledgebase system for Sphinx',
    long_description=long_description,
    url='https://github.com/pauleveritt/kaybee',
    author='Paul Everitt',
    author_email='pauleveritt@me.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Documentation',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='sphinx blog website',
    packages=find_packages(exclude=['docs', 'tests']),
    install_requires=[
        'sphinx',
        'dectate',
        'ruamel.yaml',
        'pydantic',
        'importscan',
        'pydash',
    ],
)



"""

Integration-oriented fixtures for sphinx.testing of
generated HTML.

"""

import json
import os
from pathlib import Path
from shutil import rmtree

import pytest
from bs4 import BeautifulSoup
from sphinx.testing.path import path

pytest_plugins = 'sphinx.testing.fixtures'


@pytest.fixture(scope='session')
def remove_sphinx_projects(sphinx_test_tempdir):
    # Even upon exception, remove any directory from temp area
    # which looks like a Sphinx project. This ONLY runs once.
    roots_path = Path(sphinx_test_tempdir)
    for d in roots_path.iterdir():
        if d.is_dir():
            build_dir = Path(d, '_build')
            if build_dir.exists():
                # This directory is a Sphinx project, remove it
                rmtree(str(d))

    yield


@pytest.fixture()
def rootdir(remove_sphinx_projects):
    roots = path(os.path.dirname(__file__) or '.').abspath() / 'roots'
    yield roots


@pytest.fixture()
def content(app):
    app.build()
    yield app


@pytest.fixture()
def page(content, request):
    pagename = request.param
    c = (content.outdir / pagename).text()

    yield BeautifulSoup(c, 'html5lib')


@pytest.fixture()
def json_page(content, request):
    pagename = request.param
    c = (content.outdir / pagename).text()

    yield json.loads(c)



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='resourcetype')


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestResourceTypeDebug:

    @pytest.mark.parametrize('docname, parents', [
        ('index', []),
        ('about', ['index']),
        ('folder1/index', ['index']),
        ('folder1/about', ['folder1/index', 'index']),
        ('folder1/subfolder2/index', ['folder1/index', 'index']),
        ('folder1/subfolder2/about',
         ['folder1/subfolder2/index', 'folder1/index', 'index']),
    ])
    def test_page(self, json_page, docname, parents):
        resources = json_page['resources']
        r = 'kaybee.plugins.resources.resource.Resource'

        # Check the config
        assert r == resources['config']['resource']

        # Now values
        v = resources['values']
        assert 6 == len(v.values())
        resource = v[docname]
        assert docname == resource['docname']

        # Now the custom debugdumper for this test root which serializes
        # parents and repr
        assert parents == resource['parent_docnames']
        assert docname == resource['repr']



import pytest

pytestmark = pytest.mark.sphinx('html', testroot='widgets1')


@pytest.mark.parametrize('page', ['index.html', ], indirect=True)
class TestWidgets1:

    def test_widgets1(self, page):
        # The genericpage doesn't have an acquire for template, so just use
        h1 = page.find('h1').contents[0].strip()
        assert 'Hello World' == h1

        hello = page.find(id='hello').contents[0].strip()
        assert 'Hello Custom Widget Template' == hello
        docname = page.find(id='resource_docname').contents[0].strip()
        assert 'index' == docname

    def test_widgets2(self, page):
        # The genericpage doesn't have an acquire for template, so just use
        h1 = page.find('h1').contents[0].strip()
        assert 'Hello World' == h1

        hello = page.find(id='hello').contents[0].strip()
        assert 'Hello Custom Widget Template' == hello
        docname = page.find(id='resource_docname').contents[0].strip()
        assert 'index' == docname


@pytest.mark.parametrize('json_page', ['debug_dump.json', ], indirect=True)
class TestWidgets1Debug:

    def test_settings(self, json_page):
        assert 'widgets' in json_page
        widgets = json_page['widgets']
        assert 'config' in widgets

        # widgets1 contains one registered handler, skip any registered
        config = widgets['config']
        assert 'videoplayer' in config

        # one value in widgets
        values = widgets['values']
        assert 2 == len(values)
        assert 'index-widgets1hello' in values
        assert 'index-widgets2hello' in values
        index = values['index-widgets1hello']
        assert 'index' == index['docname']
        assert 'props' in index
        props = index['props']
        assert 'widgets1_hello' == props['template']
